/* 
 * (c) 2014 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib/gi18n.h>
#include "file.h"
#include "dibuja.h"
#include "util.h"
#include "gdkpixbuf-drawable.h"


static gint file_status = FILE_NEW;

void file_set_status(gint new_status)
{
	file_status = new_status;
	if(file_status & FILE_NEED_SAVE){
		dibuja_set_title();
	}
}

gint file_get_status()
{
	return file_status;
}

char *file_def_save_folder(void)
{
	char *folder = g_strdup(g_get_user_special_dir(G_USER_DIRECTORY_PICTURES));

	if(!(g_file_test(folder, G_FILE_TEST_IS_DIR) &&
	   g_file_test(folder, G_FILE_TEST_EXISTS))){
	   
		g_free(folder);
		folder = g_strdup((gchar *)g_get_home_dir());
	}

	return folder;
}

char *file_get_cur_dir(void)
{
	char *dir = g_path_get_dirname(dibuja_get_filename());

	if(!dir) dir = g_strdup(".");

	return dir;
}

// TODO: Show file extensions
gchar *file_open_dialog(GtkWidget *widget)
{
	GtkWidget *dialog = NULL;
	GtkFileFilter *filter;
	gchar *filename = NULL;
	gchar *cur_dir = NULL;
											/* Dialog title */
	dialog = gtk_file_chooser_dialog_new (_("Open File"),
										  GTK_WINDOW(gtk_widget_get_toplevel(widget)),
										  GTK_FILE_CHOOSER_ACTION_OPEN,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
										  NULL);
	
	filter = gtk_file_filter_new ();

	gtk_file_filter_set_name(filter, _("Supported Formats"));
	gtk_file_filter_add_pixbuf_formats(filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);

	cur_dir = file_get_cur_dir();
	if(cur_dir){
		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), cur_dir);
		g_free(cur_dir);
	}

	if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT){
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	}

	gtk_widget_destroy (dialog);

	return filename;
}

GdkPixbuf *file_load_pixbuf(char *filename, gboolean warn)
{
	GError *error = NULL;
	GdkPixbuf *pixbuf = NULL;

	pixbuf = gdk_pixbuf_new_from_file((const char *)filename, &error);
	if(NULL != error){
		g_message ("[%d] Unable to read pixbuf file: %s.", __LINE__, error->message);
		if(warn)
			message_box_error(error->message);
		g_error_free(error);
	}
	else if(!GDK_IS_PIXBUF(pixbuf)) {
		g_message ("[%d] Unable to read pixbuf file: %s\n", __LINE__, filename);
		if(warn)
			message_box_error("Unable to read pixbuf file: %s\n", filename);
	}



	return pixbuf;
}

int file_open_pixbuf(char *filename)
{
	GdkPixbufFormat *pbformat;
	gchar *format = NULL;
	gchar *pbfilename = NULL;
	GdkPixbuf *pixbuf = NULL;
	int ret = 0;

	if(!filename) return 0;

	if(g_strstr_len(filename, -1, "://")){
		pbfilename = g_filename_from_uri((const gchar *)filename, NULL, NULL);
	}
	else{
		pbfilename = g_strdup(filename);
	}

	pixbuf = file_load_pixbuf(pbfilename, TRUE);
	if(GDK_IS_PIXBUF(pixbuf)){
		pbformat = gdk_pixbuf_get_file_info(pbfilename, NULL, NULL);
		format = g_strdup(gdk_pixbuf_format_get_name(pbformat));

		dibuja_new_file_from_pixbuf(pixbuf, pbfilename, format);
		g_object_unref(pixbuf);
	
		file_status = FILE_EXISTING;
		ret = 1;
	}
	if(format)
		g_free(format);
	if(pbfilename)
		g_free(pbfilename);

	return ret;
}

int file_open(GtkWidget *widget)
{
	gchar *filename = NULL;
	int ret = 0;

	if(file_status & FILE_NEED_SAVE){
		int resp = message_box_yes_no_cancel(_("The current file \"%s\" needs saving.\n"
											"Save?"),
											 dibuja_get_filename());
		if(GTK_RESPONSE_CANCEL == resp)
			return ret;
	}

	filename = file_open_dialog(widget);
	if(NULL != filename){
		GtkWidget *main_wnd = GTK_WIDGET(gtk_widget_get_toplevel(dibuja_get_canvas()));
		GdkCursor *cursor = gdk_window_get_cursor(main_wnd->window);

		canvas_set_wait_cursor(main_wnd);
		canvas_set_wait_cursor(dibuja_get_canvas());

		ret = file_open_pixbuf(filename);
		g_free(filename);

		canvas_set_cursor(dibuja_get_canvas(), DIBUJA_CURSOR);
		gdk_window_set_cursor(main_wnd->window, cursor);
	}

	return ret;
}

int file_save_pixbuf(char *filename, char *format)
{
	GdkPixbuf *pixbuf;
	cairo_surface_t *surface;

	surface = dibuja_get_surface();
	pixbuf = gdk_pixbuf_get_from_surface(surface, 0, 0, 
								cairo_image_surface_get_width(surface),
								cairo_image_surface_get_height(surface));

	gdk_pixbuf_save (pixbuf, filename, format, NULL, NULL);
	g_object_unref(pixbuf);
	return 0;
}

int file_save_dialog(GtkWidget *widget, gboolean save_as)
{
	GtkWidget *dialog;
	// TODO: get current user
	gchar *default_folder_for_saving = NULL;
	gchar *filename = NULL;
	gchar *format = NULL;
	gchar **extensions = NULL;
	GdkPixbufFormat *pbformat;
	int response = 0;
	char by_extension[] = N_("By extension");

	if((file_status & FILE_NEW) || save_as){
		GtkFileFilter *ff = NULL;

		default_folder_for_saving = file_def_save_folder();
		if(!default_folder_for_saving){
			g_warning("Cannot get default folder for saving.\n");
			return 0;
		}
		if('\0' == default_folder_for_saving){
			g_warning("Default folder for saving is NULL\n");
			return 0;
		}

		dialog = gtk_file_chooser_dialog_new(!save_as ? _("Save File") : _("Save File As"),
											 GTK_WINDOW(gtk_widget_get_toplevel(widget)),
											 GTK_FILE_CHOOSER_ACTION_SAVE,
											 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
											 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
											 NULL);


		file_chooser_dialog_add_writable_pixbuf_formats(GTK_FILE_CHOOSER (dialog), _(by_extension));
		gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);

		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), dibuja_get_filename());

		if(file_status & FILE_NEW){
			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog),
												default_folder_for_saving);
			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER (dialog),
											  dibuja_get_filename());
		}

		if ((response = gtk_dialog_run (GTK_DIALOG (dialog))) != GTK_RESPONSE_ACCEPT){

			gtk_widget_destroy (dialog);
			if(default_folder_for_saving){
				g_free(default_folder_for_saving);
			}
			return 0;
		}

		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

		ff = gtk_file_chooser_get_filter(GTK_FILE_CHOOSER (dialog));
		if(g_strstr_len(gtk_file_filter_get_name(ff), -1, by_extension))
			pbformat = get_gdk_pixbuf_format_by_extension (filename);
		else
			pbformat = (GdkPixbufFormat *)g_object_get_data(G_OBJECT(ff), "pixbuf-format");

		if(NULL !=  pbformat){
			extensions = gdk_pixbuf_format_get_extensions(pbformat);
			format = g_strdup(extensions[0]);
			g_strfreev(extensions);
		}
		else{
			format = g_strdup("png");
		}

		gtk_widget_destroy (dialog);
	}
	else{
		filename = g_strdup(dibuja_get_filename());
		format = g_strdup(dibuja_get_file_format());
		if(!format){
			g_warning("[%s: %d] Format == NULL\n", __FILE__, __LINE__);
		}
	}


	file_save_pixbuf(filename, format);
	file_status = FILE_EXISTING;
	dibuja_set_filename(filename, format);
	dibuja_set_title();
	if(default_folder_for_saving){
		g_free(default_folder_for_saving);
	}
	g_free (filename);
	g_free (format);

	return 1;
}

void file_new_file(void)
{
	if(file_status & FILE_NEED_SAVE){
		int ret = message_box_yes_no_cancel(_("The current file \"%s\" needs saving.\n"
											"Save?"),
											dibuja_get_filename());
		if(GTK_RESPONSE_CANCEL == ret)
			return;
	}
	dibuja_new_file(NULL, NULL, CANVAS_DEF_WIDTH, CANVAS_DEF_HEIGHT);
}

void file_new_file_at_size(void)
{
	gint result;
	GtkWidget *dialog = get_widget("dialog_new_image");
	GtkWidget *spin_width = get_widget("new_image_width_spin");
	GtkWidget *spin_height = get_widget("new_image_height_spin");
	GtkComboBox *combo_box = GTK_COMBO_BOX(get_widget("filesize_type_combo"));
	gdouble width, height;
	gint meas;

	if(file_status & FILE_NEED_SAVE){
		int ret = message_box_yes_no_cancel(_("The current file \"%s\" needs saving.\n"
											"Save?"),
											dibuja_get_filename());
		if(GTK_RESPONSE_CANCEL == ret)
			return;
	}

	width = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_width));
	if(width < 1.0){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_width), 600.0);
	}
	height = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_height));
	if(height < 1.0){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_height), 400.0);
	}
	/* On reshow, one of the buttons has the focus */
	gtk_widget_grab_focus(spin_width);

	result = gtk_dialog_run (GTK_DIALOG (dialog));
	if(GTK_RESPONSE_OK == result){
		width = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_width));
		height = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_height));
		meas = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_box));
		if(width > 0 && height > 0){
			gdouble xres, yres;
			gdouble mm_per_in = 25.4;

			gimp_get_screen_resolution(gdk_display_get_default_screen (gdk_display_get_default ()),
							   &xres, &yres);
			switch(meas){
				case 0:
					//pixels
					{
					int w, h;
					width = (double)(w = (int)width);
					height = (double)(h = (int)height);
					}
					break;
				case 1:
					// Convert inches to pixels
					width = ROUND(width * xres);
					height = ROUND(height * yres);
					break;
				case 2:
					// Convert millimeters to pixels
					width = ROUND((double)width / mm_per_in * xres);
					height = ROUND((double)height / mm_per_in * yres);
					break;
				default:
					return;
			}
			dibuja_new_file(NULL, NULL, width, height);
		}
	}

	gtk_widget_hide (dialog);
}














